use std::io;
use std::vec::Vec;

use crate::utils;

pub fn execute() -> Result<(), io::Error> {
    let mut stacks1: [Vec<char>; 9] = Default::default();
    let mut stacks2: [Vec<char>; 9] = Default::default();
    
    for line in utils::read_lines("input/day05.txt")? {
        let text = line?;

        match text {
            x if x.contains("[") => {
                initialize_stacks(&mut stacks1, &x);
                initialize_stacks(&mut stacks2, &x);
            },
            x if x.contains("move") => {
                move_stacks(&mut stacks1, &x, 1);
                move_stacks(&mut stacks2, &x, 2);
            },
            _ => {},
        };
    }

    utils::print_step(1, &stacks1.iter().map(|x| x.last().unwrap()).collect::<String>());
    utils::print_step(1, &stacks2.iter().map(|x| x.last().unwrap()).collect::<String>());
    
    Ok(())
}

fn initialize_stacks(stacks: &mut [Vec<char>], text: &str) {
    let mut iter = text.chars();
    iter.next(); // discard the first value
    for (i, stack) in iter.step_by(4).zip(stacks) {
        if i.is_alphabetic() { stack.insert(0, i); }
    }
}

fn move_stacks(stacks: &mut [Vec<char>], text: &str, step: u8) {
    let fields = text.split(' ').collect::<Vec<_>>();
    let count = fields[1].parse::<usize>().unwrap();
    let start = fields[3].parse::<usize>().unwrap();
    let end = fields[5].parse::<usize>().unwrap();
    let mut crane = vec![];

    for _ in 0..count {
        let item = stacks[start-1].pop().unwrap();
        match step {
            1 => stacks[end-1].push(item),
            2 => crane.push(item),
            _ => panic!("Bad step provided to function."),
        }
    }

    // if we're doing step 1, we're done
    if crane.is_empty() { return; }

    // step 2 logic
    while let Some(item) = crane.pop() {
        stacks[end-1].push(item);
    }
}
